<?php

namespace Mollie\Bundle\PaymentBundle\IntegrationCore\BusinessLogic\Http\DTO;

/**
 * Class PaymentMethod
 *
 * @package Mollie\Bundle\PaymentBundle\IntegrationCore\BusinessLogic\Http\DTO
 */
class PaymentMethod extends BaseDto
{
    /**
     * @var string
     */
    protected $resource;
    /**
     * @var string
     */
    protected $id;
    /**
     * @var string
     */
    protected $description;
    /**
     * @var Image
     */
    protected $image;
    /**
     * @var Issuer[]
     */
    protected $issuers = array();

    /**
     * {@inheritdoc}
     */
    public static function fromArray(array $raw)
    {
        $result = new static();

        $result->resource = static::getValue($raw, 'resource');
        $result->id = static::getValue($raw, 'id');
        $result->description = static::getValue($raw, 'description');
        $result->image = Image::fromArray(static::getValue($raw, 'image', array()));
        $result->issuers = !empty($raw['issuers']) ? Issuer::fromArrayBatch($raw['issuers']) : array();

        return $result;
    }

    /**
     * {@inheritdoc}
     */
    public function toArray()
    {
        $issuers = array();
        foreach ($this->issuers as $issuer) {
            $issuers[] = $issuer->toArray();
        }

        return array(
            'resource' => $this->resource,
            'id' => $this->id,
            'description' => $this->description,
            'image' => $this->image->toArray(),
            'issuers' => $issuers,
        );
    }

    /**
     * Returns payment methods in format
     *  method_1, method_2, ...
     *
     * @param PaymentMethod[] $paymentMethods
     *
     * @return string
     */
    public static function listPaymentMethodsAsString(array $paymentMethods)
    {
        $descriptions = array();
        foreach ($paymentMethods as $paymentMethod) {
            $descriptions[] = $paymentMethod->getDescription();
        }

        return implode(', ', $descriptions);
    }

    /**
     * @return string
     */
    public function getResource()
    {
        return $this->resource;
    }

    /**
     * @param string $resource
     */
    public function setResource($resource)
    {
        $this->resource = $resource;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return Image
     */
    public function getImage()
    {
        return $this->image;
    }

    /**
     * @param Image $image
     */
    public function setImage($image)
    {
        $this->image = $image;
    }

    /**
     * @return Issuer[]
     */
    public function getIssuers()
    {
        return $this->issuers;
    }

    /**
     * @param Issuer[] $issuers
     */
    public function setIssuers($issuers)
    {
        $this->issuers = $issuers;
    }
}
